@@ -31,6 +31,7 @@ const X86: &str = "x86";
 const X86_64: &str = "x86_64";
 const AARCH64: &str = "aarch64";
 const ARM: &str = "arm";
+const WASM32: &str = "wasm32";
 
 #[rustfmt::skip]
 const RING_SRCS: &[(&[&str], &str)] = &[
@@ -93,6 +94,7 @@ const RING_SRCS: &[(&[&str], &str)] = &[
     (&[AARCH64], "crypto/fipsmodule/bn/asm/armv8-mont.pl"),
     (&[AARCH64], "crypto/fipsmodule/ec/asm/p256-armv8-asm.pl"),
     (&[AARCH64], "crypto/fipsmodule/modes/asm/ghash-neon-armv8.pl"),
+    (&[AARCH64], "crypto/fipsmodule/modes/asm/aesv8-gcm-armv8.pl"),
     (&[AARCH64], SHA512_ARMV8),
 ];
 
@@ -111,18 +113,11 @@ fn cpp_flags(compiler: &cc::Tool) -> &'static [&'static str] {
         static NON_MSVC_FLAGS: &[&str] = &[
             "-fvisibility=hidden",
             "-std=c1x", // GCC 4.6 requires "c1x" instead of "c11"
-            "-pedantic",
             "-Wall",
-            "-Wextra",
             "-Wbad-function-cast",
             "-Wcast-align",
             "-Wcast-qual",
             "-Wconversion",
-            "-Wenum-compare",
-            "-Wfloat-equal",
-            "-Wformat=2",
-            "-Winline",
-            "-Winvalid-pch",
             "-Wmissing-field-initializers",
             "-Wmissing-include-dirs",
             "-Wnested-externs",
@@ -133,7 +128,6 @@ fn cpp_flags(compiler: &cc::Tool) -> &'static [&'static str] {
             "-Wstrict-prototypes",
             "-Wundef",
             "-Wuninitialized",
-            "-Wwrite-strings",
         ];
         NON_MSVC_FLAGS
     } else {
@@ -162,63 +156,63 @@ fn cpp_flags(compiler: &cc::Tool) -> &'static [&'static str] {
 const ASM_TARGETS: &[AsmTarget] = &[
     AsmTarget {
         oss: LINUX_ABI,
-        arch: "aarch64",
+        arch: AARCH64,
         perlasm_format: "linux64",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: LINUX_ABI,
-        arch: "arm",
+        arch: ARM,
         perlasm_format: "linux32",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: LINUX_ABI,
-        arch: "x86",
+        arch: X86,
         perlasm_format: "elf",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: LINUX_ABI,
-        arch: "x86_64",
+        arch: X86_64,
         perlasm_format: "elf",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: MACOS_ABI,
-        arch: "aarch64",
+        arch: AARCH64,
         perlasm_format: "ios64",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: MACOS_ABI,
-        arch: "x86_64",
+        arch: X86_64,
         perlasm_format: "macosx",
         asm_extension: "S",
         preassemble: false,
     },
     AsmTarget {
         oss: &[WINDOWS],
-        arch: "x86",
+        arch: X86,
         perlasm_format: "win32n",
         asm_extension: "asm",
         preassemble: true,
     },
     AsmTarget {
         oss: &[WINDOWS],
-        arch: "x86_64",
+        arch: X86_64,
         perlasm_format: "nasm",
         asm_extension: "asm",
         preassemble: true,
     },
     AsmTarget {
         oss: &[WINDOWS],
-        arch: "aarch64",
+        arch: AARCH64,
         perlasm_format: "win64",
         asm_extension: "S",
         preassemble: false,
@@ -264,8 +258,9 @@ const LINUX_ABI: &[&str] = &[
 
 /// Operating systems that have the same ABI as macOS on every architecture
 /// mentioned in `ASM_TARGETS`.
-const MACOS_ABI: &[&str] = &["ios", "macos", "tvos"];
+const MACOS_ABI: &[&str] = &["ios", MACOS, "tvos"];
 
+const MACOS: &str = "macos";
 const WINDOWS: &str = "windows";
 
 /// Read an environment variable and tell Cargo that we depend on it.
@@ -588,7 +583,7 @@ fn configure_cc(c: &mut cc::Build, target: &Target, include_dir: &Path) {
         let _ = c.flag(f);
     }
 
-    if target.os.as_str() == "macos" {
+    if target.os.as_str() == MACOS {
         // ``-gfull`` is required for Darwin's |-dead_strip|.
         let _ = c.flag("-gfull");
     } else if !compiler.is_like_msvc() {
@@ -602,8 +597,7 @@ fn configure_cc(c: &mut cc::Build, target: &Target, include_dir: &Path) {
     // Allow cross-compiling without a target sysroot for these targets.
     //
     // poly1305_vec.c requires <emmintrin.h> which requires <stdlib.h>.
-    if (target.arch == "wasm32")
-        || (target.os == "linux" && target.is_musl && target.arch != "x86_64")
+    if (target.arch == WASM32) || (target.os == "linux" && target.is_musl && target.arch != X86_64)
     {
         if let Ok(compiler) = c.try_get_compiler() {
             // TODO: Expand this to non-clang compilers in 0.17.0 if practical.
@@ -634,8 +628,8 @@ fn cc_asm(b: &cc::Build, file: &Path, out_file: &Path) -> Command {
 
 fn nasm(file: &Path, arch: &str, include_dir: &Path, out_file: &Path) -> Command {
     let oformat = match arch {
-        "x86_64" => "win64",
-        "x86" => "win32",
+        x if x == X86_64 => "win64",
+        x if x == X86 => "win32",
         _ => panic!("unsupported arch: {}", arch),
     };
 
@@ -742,7 +736,7 @@ fn perlasm(src_dst: &[(PathBuf, PathBuf)], asm_target: &AsmTarget) {
             src.to_string_lossy().into_owned(),
             asm_target.perlasm_format.to_owned(),
         ];
-        if asm_target.arch == "x86" {
+        if asm_target.arch == X86 {
             args.push("-fPIC".into());
             args.push("-DOPENSSL_IA32_SSE2".into());
         }
@@ -943,6 +937,8 @@ fn prefix_all_symbols(pp: char, prefix_prefix: &str, prefix: &str) -> String {
         "gcm_init_avx",
         "gcm_init_clmul",
         "gcm_init_neon",
+        "aes_gcm_enc_kernel",
+        "aes_gcm_dec_kernel",
         "k25519Precomp",
         "limbs_mul_add_limb",
         "little_endian_bytes_from_scalar",
