@@ -17,10 +17,7 @@ use super::{
     block::{Block, BLOCK_LEN},
     gcm, shift, Aad, Nonce, Tag,
 };
-use crate::{
-    aead, cpu, error,
-    polyfill::{self},
-};
+use crate::{aead, cpu, error, polyfill::usize_from_u64_saturated};
 use core::ops::RangeFrom;
 
 /// AES-128 in GCM mode with 128-bit tags and 96 bit nonces.
@@ -30,7 +27,6 @@ pub static AES_128_GCM: aead::Algorithm = aead::Algorithm {
     seal: aes_gcm_seal,
     open: aes_gcm_open,
     id: aead::AlgorithmID::AES_128_GCM,
-    max_input_len: AES_GCM_MAX_INPUT_LEN,
 };
 
 /// AES-256 in GCM mode with 128-bit tags and 96 bit nonces.
@@ -40,7 +36,6 @@ pub static AES_256_GCM: aead::Algorithm = aead::Algorithm {
     seal: aes_gcm_seal,
     open: aes_gcm_open,
     id: aead::AlgorithmID::AES_256_GCM,
-    max_input_len: AES_GCM_MAX_INPUT_LEN,
 };
 
 #[derive(Clone)]
@@ -63,28 +58,35 @@ fn init(
     cpu_features: cpu::Features,
 ) -> Result<aead::KeyInner, error::Unspecified> {
     let aes_key = aes::Key::new(key, variant, cpu_features)?;
-    let gcm_key = gcm::Key::new(aes_key.encrypt_block(Block::zero()), cpu_features);
+    let gcm_key = gcm::Key::new(
+        aes_key.encrypt_block(Block::zero(), cpu_features),
+        cpu_features,
+    );
     Ok(aead::KeyInner::AesGcm(Key { gcm_key, aes_key }))
 }
 
 const CHUNK_BLOCKS: usize = 3 * 1024 / 16;
 
-fn aes_gcm_seal(key: &aead::KeyInner, nonce: Nonce, aad: Aad<&[u8]>, in_out: &mut [u8]) -> Tag {
+fn aes_gcm_seal(
+    key: &aead::KeyInner,
+    nonce: Nonce,
+    aad: Aad<&[u8]>,
+    in_out: &mut [u8],
+    cpu_features: cpu::Features,
+) -> Result<Tag, error::Unspecified> {
     let Key { gcm_key, aes_key } = match key {
         aead::KeyInner::AesGcm(key) => key,
         _ => unreachable!(),
     };
 
+    let mut auth = gcm::Context::new(gcm_key, aad, in_out.len(), cpu_features)?;
+
     let mut ctr = Counter::one(nonce);
     let tag_iv = ctr.increment();
 
-    let total_in_out_len = in_out.len();
-    let aad_len = aad.0.len();
-    let mut auth = gcm::Context::new(gcm_key, aad);
-
     #[cfg(target_arch = "x86_64")]
     let in_out = {
-        if !aes_key.is_aes_hw() || !auth.is_avx2() {
+        if !aes_key.is_aes_hw(cpu_features) || !auth.is_avx() {
             in_out
         } else {
             use crate::c;
@@ -117,6 +119,42 @@ fn aes_gcm_seal(key: &aead::KeyInner, nonce: Nonce, aad: Aad<&[u8]>, in_out: &mu
         }
     };
 
+    #[cfg(target_arch = "aarch64")]
+    let in_out = {
+        if !aes_key.is_aes_hw(cpu_features) || !auth.is_clmul() {
+            in_out
+        } else {
+            let whole_block_bits = auth.in_out_whole_block_bits();
+            if whole_block_bits.as_bits() > 0 {
+                use crate::{bits::BitLength, c};
+                let (htable, xi) = auth.inner();
+                prefixed_extern! {
+                    fn aes_gcm_enc_kernel(
+                        input: *const u8,
+                        in_bits: BitLength<c::size_t>,
+                        output: *mut u8,
+                        Xi: &mut gcm::Xi,
+                        ivec: &mut Counter,
+                        key: &aes::AES_KEY,
+                        Htable: &gcm::HTable);
+                }
+                unsafe {
+                    aes_gcm_enc_kernel(
+                        in_out.as_ptr(),
+                        whole_block_bits,
+                        in_out.as_mut_ptr(),
+                        xi,
+                        &mut ctr,
+                        aes_key.inner_less_safe(),
+                        htable,
+                    )
+                }
+            }
+
+            &mut in_out[whole_block_bits.as_usize_bytes_rounded_up()..]
+        }
+    };
+
     let (whole, remainder) = {
         let in_out_len = in_out.len();
         let whole_len = in_out_len - (in_out_len % BLOCK_LEN);
@@ -124,20 +162,20 @@ fn aes_gcm_seal(key: &aead::KeyInner, nonce: Nonce, aad: Aad<&[u8]>, in_out: &mu
     };
 
     for chunk in whole.chunks_mut(CHUNK_BLOCKS * BLOCK_LEN) {
-        aes_key.ctr32_encrypt_within(chunk, 0.., &mut ctr);
+        aes_key.ctr32_encrypt_within(chunk, 0.., &mut ctr, cpu_features);
         auth.update_blocks(chunk);
     }
 
     if !remainder.is_empty() {
         let mut input = Block::zero();
         input.overwrite_part_at(0, remainder);
-        let mut output = aes_key.encrypt_iv_xor_block(ctr.into(), input);
+        let mut output = aes_key.encrypt_iv_xor_block(ctr.into(), input, cpu_features);
         output.zero_from(remainder.len());
         auth.update_block(output);
         remainder.copy_from_slice(&output.as_ref()[..remainder.len()]);
     }
 
-    finish(aes_key, auth, tag_iv, aad_len, total_in_out_len)
+    Ok(finish(aes_key, auth, tag_iv))
 }
 
 fn aes_gcm_open(
@@ -146,25 +184,29 @@ fn aes_gcm_open(
     aad: Aad<&[u8]>,
     in_out: &mut [u8],
     src: RangeFrom<usize>,
-) -> Tag {
+    cpu_features: cpu::Features,
+) -> Result<Tag, error::Unspecified> {
     let Key { gcm_key, aes_key } = match key {
         aead::KeyInner::AesGcm(key) => key,
         _ => unreachable!(),
     };
 
+    let mut auth = {
+        let unprefixed_len = in_out
+            .len()
+            .checked_sub(src.start)
+            .ok_or(error::Unspecified)?;
+        gcm::Context::new(gcm_key, aad, unprefixed_len, cpu_features)
+    }?;
+
     let mut ctr = Counter::one(nonce);
     let tag_iv = ctr.increment();
 
-    let aad_len = aad.0.len();
-    let mut auth = gcm::Context::new(gcm_key, aad);
-
     let in_prefix_len = src.start;
 
-    let total_in_out_len = in_out.len() - in_prefix_len;
-
     #[cfg(target_arch = "x86_64")]
     let in_out = {
-        if !aes_key.is_aes_hw() || !auth.is_avx2() {
+        if !aes_key.is_aes_hw(cpu_features) || !auth.is_avx() {
             in_out
         } else {
             use crate::c;
@@ -197,6 +239,43 @@ fn aes_gcm_open(
         }
     };
 
+    #[cfg(target_arch = "aarch64")]
+    let in_out = {
+        if !aes_key.is_aes_hw(cpu_features) || !auth.is_clmul() {
+            in_out
+        } else {
+            let whole_block_bits = auth.in_out_whole_block_bits();
+            if whole_block_bits.as_bits() > 0 {
+                use crate::{bits::BitLength, c};
+                let (htable, xi) = auth.inner();
+                prefixed_extern! {
+                    fn aes_gcm_dec_kernel(
+                        input: *const u8,
+                        in_bits: BitLength<c::size_t>,
+                        output: *mut u8,
+                        Xi: &mut gcm::Xi,
+                        ivec: &mut Counter,
+                        key: &aes::AES_KEY,
+                        Htable: &gcm::HTable);
+                }
+
+                unsafe {
+                    aes_gcm_dec_kernel(
+                        in_out[src.clone()].as_ptr(),
+                        whole_block_bits,
+                        in_out.as_mut_ptr(),
+                        xi,
+                        &mut ctr,
+                        aes_key.inner_less_safe(),
+                        htable,
+                    )
+                }
+            }
+
+            &mut in_out[whole_block_bits.as_usize_bytes_rounded_up()..]
+        }
+    };
+
     let whole_len = {
         let in_out_len = in_out.len() - in_prefix_len;
         in_out_len - (in_out_len % BLOCK_LEN)
@@ -218,6 +297,7 @@ fn aes_gcm_open(
                 &mut in_out[output..][..(chunk_len + in_prefix_len)],
                 in_prefix_len..,
                 &mut ctr,
+                cpu_features,
             );
             output += chunk_len;
             input += chunk_len;
@@ -229,56 +309,29 @@ fn aes_gcm_open(
         let mut input = Block::zero();
         input.overwrite_part_at(0, remainder);
         auth.update_block(input);
-        aes_key.encrypt_iv_xor_block(ctr.into(), input)
+        aes_key.encrypt_iv_xor_block(ctr.into(), input, cpu_features)
     });
 
-    finish(aes_key, auth, tag_iv, aad_len, total_in_out_len)
+    Ok(finish(aes_key, auth, tag_iv))
 }
 
-fn finish(
-    aes_key: &aes::Key,
-    mut gcm_ctx: gcm::Context,
-    tag_iv: aes::Iv,
-    aad_len: usize,
-    in_out_len: usize,
-) -> Tag {
-    // Authenticate the final block containing the input lengths.
-    let aad_bits = polyfill::u64_from_usize(aad_len) << 3;
-    let ciphertext_bits = polyfill::u64_from_usize(in_out_len) << 3;
-    gcm_ctx.update_block(Block::from(
-        [aad_bits, ciphertext_bits].map(u64::to_be_bytes),
-    ));
-
+fn finish(aes_key: &aes::Key, gcm_ctx: gcm::Context, tag_iv: aes::Iv) -> Tag {
     // Finalize the tag and return it.
-    gcm_ctx.pre_finish(|pre_tag| {
-        let encrypted_iv = aes_key.encrypt_block(tag_iv.into_block_less_safe());
+    gcm_ctx.pre_finish(|pre_tag, cpu_features| {
+        let encrypted_iv = aes_key.encrypt_block(tag_iv.into_block_less_safe(), cpu_features);
         let tag = pre_tag ^ encrypted_iv;
         Tag(*tag.as_ref())
     })
 }
 
-const AES_GCM_MAX_INPUT_LEN: u64 = super::max_input_len(BLOCK_LEN, 2);
+pub(super) const MAX_IN_OUT_LEN: usize = super::max_input_len(BLOCK_LEN, 2);
 
-#[cfg(test)]
-mod tests {
-    #[test]
-    fn max_input_len_test() {
-        // [NIST SP800-38D] Section 5.2.1.1. Note that [RFC 5116 Section 5.1] and
-        // [RFC 5116 Section 5.2] have an off-by-one error in `P_MAX`.
-        //
-        // [NIST SP800-38D]:
-        //    http://nvlpubs.nist.gov/nistpubs/Legacy/SP/nistspecialpublication800-38d.pdf
-        // [RFC 5116 Section 5.1]: https://tools.ietf.org/html/rfc5116#section-5.1
-        // [RFC 5116 Section 5.2]: https://tools.ietf.org/html/rfc5116#section-5.2
-        const NIST_SP800_38D_MAX_BITS: u64 = (1u64 << 39) - 256;
-        assert_eq!(NIST_SP800_38D_MAX_BITS, 549_755_813_632u64);
-        assert_eq!(
-            super::AES_128_GCM.max_input_len * 8,
-            NIST_SP800_38D_MAX_BITS
-        );
-        assert_eq!(
-            super::AES_256_GCM.max_input_len * 8,
-            NIST_SP800_38D_MAX_BITS
-        );
-    }
-}
+// [NIST SP800-38D] Section 5.2.1.1. Note that [RFC 5116 Section 5.1] and
+// [RFC 5116 Section 5.2] have an off-by-one error in `P_MAX`.
+//
+// [NIST SP800-38D]:
+//    http://nvlpubs.nist.gov/nistpubs/Legacy/SP/nistspecialpublication800-38d.pdf
+// [RFC 5116 Section 5.1]: https://tools.ietf.org/html/rfc5116#section-5.1
+// [RFC 5116 Section 5.2]: https://tools.ietf.org/html/rfc5116#section-5.2
+const _MAX_INPUT_LEN_BOUNDED_BY_NIST: () =
+    assert!(MAX_IN_OUT_LEN == usize_from_u64_saturated(((1u64 << 39) - 256) / 8));
