@@ -14,6 +14,8 @@
 
 //! ECDSA Signatures using the P-256 and P-384 curves.
 
+#![allow(clippy::cast_possible_truncation)] // XXX
+
 use super::digest_scalar::digest_scalar;
 use crate::{
     arithmetic::montgomery::*,
@@ -26,8 +28,6 @@ use crate::{
     io::der,
     limb, pkcs8, rand, sealed, signature,
 };
-use untrusted;
-
 /// An ECDSA signing algorithm.
 pub struct EcdsaSigningAlgorithm {
     curve: &'static ec::Curve,
@@ -62,6 +62,7 @@ impl sealed::Sealed for EcdsaSigningAlgorithm {}
 /// An ECDSA key pair, used for signing.
 pub struct EcdsaKeyPair {
     d: Scalar<R>,
+    nonce_key: NonceRandomKey,
     alg: &'static EcdsaSigningAlgorithm,
     public_key: PublicKey,
 }
@@ -87,7 +88,7 @@ impl EcdsaKeyPair {
         let private_key = ec::Seed::generate(alg.curve, rng, cpu::features())?;
         let public_key = private_key.compute_public_key()?;
         Ok(pkcs8::wrap_key(
-            &alg.pkcs8_template,
+            alg.pkcs8_template,
             private_key.bytes_less_safe(),
             public_key.as_ref(),
         ))
@@ -106,6 +107,7 @@ impl EcdsaKeyPair {
     pub fn from_pkcs8(
         alg: &'static EcdsaSigningAlgorithm,
         pkcs8: &[u8],
+        rng: &dyn rand::SecureRandom,
     ) -> Result<Self, error::KeyRejected> {
         let key_pair = ec::suite_b::key_pair_from_pkcs8(
             alg.curve,
@@ -113,19 +115,30 @@ impl EcdsaKeyPair {
             untrusted::Input::from(pkcs8),
             cpu::features(),
         )?;
-        Ok(Self::new(alg, key_pair))
+        Self::new(alg, key_pair, rng)
     }
 
-    /// Constructs an ECDSA key pair directly from the big-endian-encoded
-    /// private key and public key bytes.
+    /// Constructs an ECDSA key pair from the private key and public key bytes
+    ///
+    /// The private key must encoded as a big-endian fixed-length integer. For
+    /// example, a P-256 private key must be 32 bytes prefixed with leading
+    /// zeros as needed.
+    ///
+    /// The public key is encoding in uncompressed form using the
+    /// Octet-String-to-Elliptic-Curve-Point algorithm in
+    /// [SEC 1: Elliptic Curve Cryptography, Version 2.0].
     ///
     /// This is intended for use by code that deserializes key pairs. It is
-    /// recommended to use `RsaPubeyPair::from_pkcs8()` (with a PKCS#8-encoded
+    /// recommended to use `EcdsaKeyPair::from_pkcs8()` (with a PKCS#8-encoded
     /// key) instead.
+    ///
+    /// [SEC 1: Elliptic Curve Cryptography, Version 2.0]:
+    ///     http://www.secg.org/sec1-v2.pdf
     pub fn from_private_key_and_public_key(
         alg: &'static EcdsaSigningAlgorithm,
         private_key: &[u8],
         public_key: &[u8],
+        rng: &dyn rand::SecureRandom,
     ) -> Result<Self, error::KeyRejected> {
         let key_pair = ec::suite_b::key_pair_from_bytes(
             alg.curve,
@@ -133,26 +146,28 @@ impl EcdsaKeyPair {
             untrusted::Input::from(public_key),
             cpu::features(),
         )?;
-        Ok(Self::new(alg, key_pair))
+        Self::new(alg, key_pair, rng)
     }
 
-    fn new(alg: &'static EcdsaSigningAlgorithm, key_pair: ec::KeyPair) -> Self {
+    fn new(
+        alg: &'static EcdsaSigningAlgorithm,
+        key_pair: ec::KeyPair,
+        rng: &dyn rand::SecureRandom,
+    ) -> Result<Self, error::KeyRejected> {
         let (seed, public_key) = key_pair.split();
         let d = private_key::private_key_as_scalar(alg.private_key_ops, &seed);
-        let d = alg
-            .private_scalar_ops
-            .scalar_ops
-            .scalar_product(&d, &alg.private_scalar_ops.oneRR_mod_n);
+        let d = alg.private_scalar_ops.to_mont(&d);
 
-        Self {
+        let nonce_key = NonceRandomKey::new(alg, &seed, rng)?;
+        Ok(Self {
             d,
+            nonce_key,
             alg,
             public_key: PublicKey(public_key),
-        }
+        })
     }
 
-    /// Returns the signature of the `message` using a random nonce
-    /// generated by `rng`.
+    /// Returns the signature of the `message` using a random nonce generated by `rng`.
     pub fn sign(
         &self,
         rng: &dyn rand::SecureRandom,
@@ -160,15 +175,37 @@ impl EcdsaKeyPair {
     ) -> Result<signature::Signature, error::Unspecified> {
         // Step 4 (out of order).
         let h = digest::digest(self.alg.digest_alg, message);
-        self.sign_(rng, h)
+
+        // Incorporate `h` into the nonce to hedge against faulty RNGs. (This
+        // is not an approved random number generator that is mandated in
+        // the spec.)
+        let nonce_rng = NonceRandom {
+            key: &self.nonce_key,
+            message_digest: &h,
+            rng,
+        };
+
+        self.sign_digest(h, &nonce_rng)
+    }
+
+    #[cfg(test)]
+    fn sign_with_fixed_nonce_during_test(
+        &self,
+        rng: &dyn rand::SecureRandom,
+        message: &[u8],
+    ) -> Result<signature::Signature, error::Unspecified> {
+        // Step 4 (out of order).
+        let h = digest::digest(self.alg.digest_alg, message);
+
+        self.sign_digest(h, rng)
     }
 
     /// Returns the signature of message digest `h` using a "random" nonce
     /// generated by `rng`.
-    fn sign_(
+    fn sign_digest(
         &self,
-        rng: &dyn rand::SecureRandom,
         h: digest::Digest,
+        rng: &dyn rand::SecureRandom,
     ) -> Result<signature::Signature, error::Unspecified> {
         // NSA Suite B Implementer's Guide to ECDSA Section 3.4.1: ECDSA
         // Signature Generation.
@@ -203,7 +240,7 @@ impl EcdsaKeyPair {
             // XXX: iteration conut?
             // Step 1.
             let k = private_key::random_scalar(self.alg.private_key_ops, rng)?;
-            let k_inv = scalar_ops.scalar_inv_to_mont(&k);
+            let k_inv = ops.scalar_inv_to_mont(&k);
 
             // Step 2.
             let r = private_key_ops.point_mul_base(&k);
@@ -226,7 +263,7 @@ impl EcdsaKeyPair {
             // Step 6.
             let s = {
                 let dr = scalar_ops.scalar_product(&self.d, &r);
-                let e_plus_dr = scalar_sum(cops, &e, &dr);
+                let e_plus_dr = scalar_sum(cops, &e, dr);
                 scalar_ops.scalar_product(&k_inv, &e_plus_dr)
             };
             if cops.is_zero(&s) {
@@ -243,6 +280,86 @@ impl EcdsaKeyPair {
     }
 }
 
+/// Generates an ECDSA nonce in a way that attempts to protect against a faulty
+/// `SecureRandom`.
+struct NonceRandom<'a> {
+    key: &'a NonceRandomKey,
+    message_digest: &'a digest::Digest,
+    rng: &'a dyn rand::SecureRandom,
+}
+
+impl core::fmt::Debug for NonceRandom<'_> {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        f.debug_struct("NonceRandom").finish()
+    }
+}
+
+impl rand::sealed::SecureRandom for NonceRandom<'_> {
+    fn fill_impl(&self, dest: &mut [u8]) -> Result<(), error::Unspecified> {
+        // Use the same digest algorithm that will be used to digest the
+        // message. The digest algorithm's output is exactly the right size;
+        // this is checked below.
+        //
+        // XXX(perf): The single iteration will require two digest block
+        // operations because the amount of data digested is larger than one
+        // block.
+        let digest_alg = self.key.0.algorithm();
+        let mut ctx = digest::Context::new(digest_alg);
+
+        // Digest the randomized digest of the private key.
+        let key = self.key.0.as_ref();
+        ctx.update(key);
+
+        // The random value is digested between the key and the message so that
+        // the key and the message are not directly digested in the same digest
+        // block.
+        assert!(key.len() <= digest_alg.block_len() / 2);
+        {
+            let mut rand = [0u8; digest::MAX_BLOCK_LEN];
+            let rand = &mut rand[..digest_alg.block_len() - key.len()];
+            assert!(rand.len() >= dest.len());
+            self.rng.fill(rand)?;
+            ctx.update(rand);
+        }
+
+        ctx.update(self.message_digest.as_ref());
+
+        let nonce = ctx.finish();
+
+        // `copy_from_slice()` panics if the lengths differ, so we don't have
+        // to separately assert that the lengths are the same.
+        dest.copy_from_slice(nonce.as_ref());
+
+        Ok(())
+    }
+}
+
+impl<'a> sealed::Sealed for NonceRandom<'a> {}
+
+struct NonceRandomKey(digest::Digest);
+
+impl NonceRandomKey {
+    fn new(
+        alg: &EcdsaSigningAlgorithm,
+        seed: &ec::Seed,
+        rng: &dyn rand::SecureRandom,
+    ) -> Result<Self, error::KeyRejected> {
+        let mut rand = [0; digest::MAX_OUTPUT_LEN];
+        let rand = &mut rand[0..alg.curve.elem_scalar_seed_len];
+
+        // XXX: `KeyRejected` isn't the right way to model  failure of the RNG,
+        // but to fix that we'd need to break the API by changing the result type.
+        // TODO: Fix the API in the next breaking release.
+        rng.fill(rand)
+            .map_err(|error::Unspecified| error::KeyRejected::rng_failed())?;
+
+        let mut ctx = digest::Context::new(alg.digest_alg);
+        ctx.update(rand);
+        ctx.update(seed.bytes_less_safe());
+        Ok(Self(ctx.finish()))
+    }
+}
+
 impl signature::KeyPair for EcdsaKeyPair {
     type PublicKey = PublicKey;
 
@@ -266,10 +383,10 @@ fn format_rs_fixed(ops: &'static ScalarOps, r: &Scalar, s: &Scalar, out: &mut [u
     let scalar_len = ops.scalar_bytes_len();
 
     let (r_out, rest) = out.split_at_mut(scalar_len);
-    limb::big_endian_from_limbs(&r.limbs[..ops.common.num_limbs], r_out);
+    limb::big_endian_from_limbs(ops.leak_limbs(r), r_out);
 
     let (s_out, _) = rest.split_at_mut(scalar_len);
-    limb::big_endian_from_limbs(&s.limbs[..ops.common.num_limbs], s_out);
+    limb::big_endian_from_limbs(ops.leak_limbs(s), s_out);
 
     2 * scalar_len
 }
@@ -280,7 +397,7 @@ fn format_rs_asn1(ops: &'static ScalarOps, r: &Scalar, s: &Scalar, out: &mut [u8
     fn format_integer_tlv(ops: &ScalarOps, a: &Scalar, out: &mut [u8]) -> usize {
         let mut fixed = [0u8; ec::SCALAR_MAX_BYTES + 1];
         let fixed = &mut fixed[..(ops.scalar_bytes_len() + 1)];
-        limb::big_endian_from_limbs(&a.limbs[..ops.common.num_limbs], &mut fixed[1..]);
+        limb::big_endian_from_limbs(ops.leak_limbs(a), &mut fixed[1..]);
 
         // Since `a_fixed_out` is an extra byte long, it is guaranteed to start
         // with a zero.
@@ -303,7 +420,7 @@ fn format_rs_asn1(ops: &'static ScalarOps, r: &Scalar, s: &Scalar, out: &mut [u8
         assert!(value.len() < 128);
         out[1] = value.len() as u8;
 
-        out[2..][..value.len()].copy_from_slice(&value);
+        out[2..][..value.len()].copy_from_slice(value);
 
         2 + value.len()
     }
@@ -396,10 +513,12 @@ static EC_PUBLIC_KEY_P384_PKCS8_V1_TEMPLATE: pkcs8::Template = pkcs8::Template {
 
 #[cfg(test)]
 mod tests {
-    use crate::{signature, test};
+    use crate::{rand, signature, test};
 
     #[test]
     fn signature_ecdsa_sign_fixed_test() {
+        let rng = rand::SystemRandom::new();
+
         test::run(
             test_file!("ecdsa_sign_fixed_tests.txt"),
             |section, test_case| {
@@ -423,10 +542,13 @@ mod tests {
                 };
 
                 let private_key =
-                    signature::EcdsaKeyPair::from_private_key_and_public_key(alg, &d, &q).unwrap();
+                    signature::EcdsaKeyPair::from_private_key_and_public_key(alg, &d, &q, &rng)
+                        .unwrap();
                 let rng = test::rand::FixedSliceRandom { bytes: &k };
 
-                let actual_result = private_key.sign(&rng, &msg).unwrap();
+                let actual_result = private_key
+                    .sign_with_fixed_nonce_during_test(&rng, &msg)
+                    .unwrap();
 
                 assert_eq!(actual_result.as_ref(), &expected_result[..]);
 
@@ -437,6 +559,8 @@ mod tests {
 
     #[test]
     fn signature_ecdsa_sign_asn1_test() {
+        let rng = rand::SystemRandom::new();
+
         test::run(
             test_file!("ecdsa_sign_asn1_tests.txt"),
             |section, test_case| {
@@ -460,10 +584,13 @@ mod tests {
                 };
 
                 let private_key =
-                    signature::EcdsaKeyPair::from_private_key_and_public_key(alg, &d, &q).unwrap();
+                    signature::EcdsaKeyPair::from_private_key_and_public_key(alg, &d, &q, &rng)
+                        .unwrap();
                 let rng = test::rand::FixedSliceRandom { bytes: &k };
 
-                let actual_result = private_key.sign(&rng, &msg).unwrap();
+                let actual_result = private_key
+                    .sign_with_fixed_nonce_during_test(&rng, &msg)
+                    .unwrap();
 
                 assert_eq!(actual_result.as_ref(), &expected_result[..]);
 
